import { syntaxTree, LRLanguage, indentNodeProp, delimitedIndent, foldNodeProp, foldInside, LanguageSupport } from '@codemirror/language';
import { completeFromList } from '@codemirror/autocomplete';
import { linter } from '@codemirror/lint';
import { LRParser } from '@lezer/lr';
import { styleTags, tags } from '@lezer/highlight';

var collection = {
	type: "HEAD",
	shortDescription: "The constraint specifies which entity collection will be searched for results.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#header"
};
var filterGroupBy = {
	type: "FILTER",
	shortDescription: "The container encapsulating filter constraint limiting the facet groups returned in facet summary.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#filter-by"
};
var attributeGreaterThanEquals = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is greater than or equals to the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-greater-than-equals"
};
var filterBy = {
	type: "FILTER",
	shortDescription: "The container encapsulating inner filter constraint into one main constraint that is required by the query.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#filter-by"
};
var priceBetween = {
	type: "FILTER",
	shortDescription: "The constraint checks if entity has price for sale within the passed range of prices (both ends are inclusive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/price#price-between"
};
var entityHaving = {
	type: "FILTER",
	shortDescription: "The container allowing to filter entities by having references to entities managed by evitaDB that  + match inner filtering constraints. This container resembles the SQL inner join clauses where the `entityHaving` + contains the filtering condition on particular join.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/references#entity-having"
};
var priceInCurrency = {
	type: "FILTER",
	shortDescription: "The constraint filters out all entities that lack selling price in specified currency.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/price#price-in-currency"
};
var userFilter = {
	type: "FILTER",
	shortDescription: "The container for constraints that are controlled by the user (client UI widgets).  + It is used mainly to distinguish between user constraint (refining the search) and program defined  + constraints (considered mandatory), when the extra results are computed.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/behavioral#user-filter"
};
var attributeContains = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the text value of that attribute contains part of passed text (case-sensitive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/string#attribute-contains"
};
var excluding = {
	type: "FILTER",
	shortDescription: "The constraint narrows hierarchy within parent constraint to exclude specified hierarchy subtrees from search.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#excluding"
};
var priceValidIn = {
	type: "FILTER",
	shortDescription: "The constraint checks if entity has selling price valid at the passed moment.",
	userDocsLink: "https://evitadb.io/documentation/filtering/price#price-valid-in"
};
var attributeLessThan = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is less than the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-less-than"
};
var priceInPriceLists = {
	type: "FILTER",
	shortDescription: "The constraint filters out all entities that lack selling price in specified price lists.  + Order of price lists also defines priority for selecting the entity selling price - the price from first price  + list in the list will be used as a selling price for the entire entity.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/price#price-in-price-lists"
};
var entityLocaleEquals = {
	type: "FILTER",
	shortDescription: "The constraint if at least one of entity locales (derived from entity attributes or associated data) equals to the passed one.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/locale#entity-locale-equals"
};
var attributeGreaterThan = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is greater than the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-greater-than"
};
var facetHaving = {
	type: "FILTER",
	shortDescription: "The container allowing to filter entities by having references to entities managed by evitaDB that  + match the inner filter constraint. This container resembles the SQL inner join clauses and works in cooperation  + with facet summary requirement.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/references#facet-having"
};
var or = {
	type: "FILTER",
	shortDescription: "The container that combines inner constraints with [logical OR](https://en.wikipedia.org/wiki/Logical_disjunction).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/logical#or"
};
var attributeEquals = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if they are both equal.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-equals"
};
var hierarchyWithinRoot = {
	type: "FILTER",
	shortDescription: "The constraint if entity is placed inside the defined hierarchy tree starting at the root of the tree (or has reference to any hierarchical entity in the tree).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#hierarchy-within-root"
};
var and = {
	type: "FILTER",
	shortDescription: "The container that combines inner constraints with [logical AND](https://en.wikipedia.org/wiki/Logical_conjunction).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/logical#and"
};
var attributeBetween = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is within the passed range (both ends are inclusive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-between"
};
var referenceHaving = {
	type: "FILTER",
	shortDescription: "The container allowing to filter entities by having references to entities managed by evitaDB that  + match the inner filter constraint. This container resembles the SQL inner join clauses.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/references#reference-having"
};
var not = {
	type: "FILTER",
	shortDescription: "The container that behaves as [logical NOT](https://en.wikipedia.org/wiki/Negation) for the inner constraint.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/logical#not"
};
var attributeIs = {
	type: "FILTER",
	shortDescription: "The constraint if value of the attribute is same as passed special value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-is"
};
var attributeLessThanEquals = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute is less than or equals to the passed value.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-less-than-equals"
};
var having = {
	type: "FILTER",
	shortDescription: "The constraint narrows hierarchy within parent constraint to include specified hierarchy subtrees from search.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#having"
};
var attributeStartsWith = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the text value of that attributes starts with passed text. (case-sensitive)",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/string#attribute-starts-with"
};
var hierarchyWithin = {
	type: "FILTER",
	shortDescription: "The constraint if entity is placed inside the defined hierarchy tree (or has reference to any hierarchical entity in the tree).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#hierarchy-within"
};
var directRelation = {
	type: "FILTER",
	shortDescription: "The constraint limits hierarchy within parent constraint to take only directly related entities into an account.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#direct-relation"
};
var attributeEndsWith = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the text value of that attribute ends with passed text (case-sensitive).",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/string#attribute-ends-with"
};
var excludingRoot = {
	type: "FILTER",
	shortDescription: "The constraint limits hierarchy within parent constraint to exclude the entities directly related to the searched root node.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/hierarchy#excluding-root"
};
var attributeInSet = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the value of that attribute  + equals to at least one of the passed values.  + The constraint is equivalent to the multiple `equals` constraints combined with logical OR.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/comparable#attribute-in-set"
};
var entityPrimaryKeyInSet = {
	type: "FILTER",
	shortDescription: "The constraint checks if primary key of the entity equals to at least one of the passed values.  + The constraint is equivalent to one or more `equals` constraints combined with logical OR.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/constant#entity-primary-key-in-set"
};
var attributeInRange = {
	type: "FILTER",
	shortDescription: "Compares value of the attribute with passed value and checks if the range value of that  + attribute contains the passed value within its limits (both ends are inclusive).  + The constraint can be used only for Range data type values.",
	userDocsLink: "https://evitadb.io/documentation/query/filtering/range#attribute-in-range"
};
var entityProperty = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned references by applying ordering constraint on referenced entity.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/reference#entity-property"
};
var entityPrimaryKeyInFilter = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities by ordering of the values specified `entityPrimaryKeysInSet` in filter.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-primary-key-order-used-in-filter"
};
var referenceProperty = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities or references by attribute specified on its reference in natural order.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/reference#reference-property"
};
var priceNatural = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities by selected price for sale.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/price#price-natural"
};
var entityPrimaryKeyExact = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities by ordering of the values specified in arguments.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-primary-key-order"
};
var attributeNatural = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities by natural ordering of the values in the specified attribute.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/comparable#attribute-natural"
};
var orderGroupBy = {
	type: "ORDER",
	shortDescription: "The container encapsulates order constraints that control the order of the facet groups in facet summary.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#order-by"
};
var orderBy = {
	type: "ORDER",
	shortDescription: "The container encapsulates inner order constraints into one main constraint that is required by the query.",
	userDocsLink: "https://evitadb.io/documentation/query/basics#order-by"
};
var attributeSetInFilter = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities by ordering of the values specified `attributeInSet` in filter sharing the same attribute name.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-attribute-value-order-used-in-filter"
};
var attributeSetExact = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities by ordering of the values specified in arguments matching the entity attribute of specified name.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/constant#exact-entity-attribute-value-order"
};
var entityGroupProperty = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned references by applying ordering constraint on referenced entity group.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/reference#entity-group-property"
};
var entityPrimaryKeyNatural = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities by primary key in specific order.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/comparable#primary-key-natural"
};
var random = {
	type: "ORDER",
	shortDescription: "The constraint sorts returned entities randomly.",
	userDocsLink: "https://evitadb.io/documentation/query/ordering/random#random"
};
var hierarchyOfReference = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computation of hierarchy statistics (how many matching children the hierarchy nodes have) of referenced hierarchical entities into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#hierarchy-of-reference"
};
var priceType = {
	type: "REQUIRE",
	shortDescription: "The constraint specifies which price type (with/without tax) will be used for handling filtering and sorting constraints.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/price#price-type"
};
var dataInLocales = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers fetching of the localized attributes or associated data in different/additional locales than the locale specified in filtering constraints (if any at all).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#data-in-locales"
};
var hierarchyOfSelf = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computation of hierarchy statistics (how many matching children the hierarchy nodes have) of same hierarchical collection into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#hierarchy-of-self"
};
var fromNode = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computing the hierarchy subtree starting at pivot node.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#from-node"
};
var distance = {
	type: "REQUIRE",
	shortDescription: "The constraint limits the traversing in stop at container at specified distance (number of nodes in path).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#distance"
};
var stopAt = {
	type: "REQUIRE",
	shortDescription: "The constraint defines the traversal stop condition that limits the scope of the returned hierarchy tree.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#stop-at"
};
var facetSummaryOfReference = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computation of facet summary of all facet in searched scope into response with custom \\fetching\\ settings for specific reference.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-summary-of-reference"
};
var level = {
	type: "REQUIRE",
	shortDescription: "The constraint limits the traversing in stop at container at specified level from root.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#level"
};
var facetGroupsDisjunction = {
	type: "REQUIRE",
	shortDescription: "Sets relation of facets in the specified groups towards facets in different groups to [logical OR](https://en.wikipedia.org/wiki/Logical_disjunction) .",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-groups-disjunction"
};
var children = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computing the hierarchy subtree starting at currently requested hierarchy node in filter by constraint.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#children"
};
var facetGroupsConjunction = {
	type: "REQUIRE",
	shortDescription: "Sets inter-facets relation within the specified groups to [logical AND](https://en.wikipedia.org/wiki/Logical_conjunction).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-groups-conjunction"
};
var fromRoot = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computing the hierarchy subtree starting at root level.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#from-root"
};
var queryTelemetry = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computation of query telemetry (explaining what operations were performed and how long they took) in extra results of the response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/debug#query-telemetry"
};
var priceHistogram = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computation of the [histogram](https://en.wikipedia.org/wiki/Histogram) of price for sale into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/histogram#price-histogram"
};
var associatedDataContent = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers fetching the entity associated data of specified names into the returned entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#associated-data-content"
};
var hierarchyContent = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers fetching parent hierarchy entity parent chain and its bodies into returned main entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#hierarchy-content"
};
var statistics = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computing the count of children for each returned hierarchy node.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#statistics"
};
var facetGroupsNegation = {
	type: "REQUIRE",
	shortDescription: "[Negates](https://en.wikipedia.org/wiki/Negation) the meaning of selected facets in specified  + facet groups in the sense that their selection would return entities that don't have any of those facets.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-groups-negation"
};
var entityFetch = {
	type: "REQUIRE",
	shortDescription: "Returns richer entities instead of just entity references (empty container returns only entity body).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#entity-fetch"
};
var require = {
	type: "REQUIRE",
	shortDescription: "The container encapsulates inner require constraints into one main constraint that is required by the query",
	userDocsLink: "https://evitadb.io/documentation/query/basics#require"
};
var entityGroupFetch = {
	type: "REQUIRE",
	shortDescription: "Returns richer group entities instead of just entity references (empty container returns only entity body).",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#entity-group-fetch"
};
var node = {
	type: "REQUIRE",
	shortDescription: "The constraint allows to locate the pivot hierarchy node.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#node"
};
var attributeContent = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers fetching the entity attributes into the returned entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#attribute-content"
};
var strip = {
	type: "REQUIRE",
	shortDescription: "The constraint specifies which strip (subset) of found entities will be returned.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/paging#strip"
};
var siblings = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computing the sibling axis for currently requested hierarchy node in filter by constraint or processed node by hierarchy parents axis.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#siblings"
};
var priceContent = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers fetching the entity prices into the returned entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#price-content"
};
var page = {
	type: "REQUIRE",
	shortDescription: "The constraint specifies which page of found entities will be returned.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/paging#page"
};
var parents = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computing the hierarchy parent axis starting at currently requested hierarchy node in filter by constraint.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/hierarchy#parents"
};
var referenceContent = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers fetching referenced entity bodies into returned main entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/fetching#reference-content"
};
var attributeHistogram = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computation of the [histogram](https://en.wikipedia.org/wiki/Histogram) of specified attributes into response.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/histogram#attribute-histogram"
};
var facetSummary = {
	type: "REQUIRE",
	shortDescription: "The constraint triggers computation of facet summary of all facet in searched scope into response with default \\fetching\\ settings for all referenced entities.",
	userDocsLink: "https://evitadb.io/documentation/query/requirements/facet#facet-summary"
};
var constraints = {
	collection: collection,
	filterGroupBy: filterGroupBy,
	attributeGreaterThanEquals: attributeGreaterThanEquals,
	filterBy: filterBy,
	priceBetween: priceBetween,
	entityHaving: entityHaving,
	priceInCurrency: priceInCurrency,
	userFilter: userFilter,
	attributeContains: attributeContains,
	excluding: excluding,
	priceValidIn: priceValidIn,
	attributeLessThan: attributeLessThan,
	priceInPriceLists: priceInPriceLists,
	entityLocaleEquals: entityLocaleEquals,
	attributeGreaterThan: attributeGreaterThan,
	facetHaving: facetHaving,
	or: or,
	attributeEquals: attributeEquals,
	hierarchyWithinRoot: hierarchyWithinRoot,
	and: and,
	attributeBetween: attributeBetween,
	referenceHaving: referenceHaving,
	not: not,
	attributeIs: attributeIs,
	attributeLessThanEquals: attributeLessThanEquals,
	having: having,
	attributeStartsWith: attributeStartsWith,
	hierarchyWithin: hierarchyWithin,
	directRelation: directRelation,
	attributeEndsWith: attributeEndsWith,
	excludingRoot: excludingRoot,
	attributeInSet: attributeInSet,
	entityPrimaryKeyInSet: entityPrimaryKeyInSet,
	attributeInRange: attributeInRange,
	entityProperty: entityProperty,
	entityPrimaryKeyInFilter: entityPrimaryKeyInFilter,
	referenceProperty: referenceProperty,
	priceNatural: priceNatural,
	entityPrimaryKeyExact: entityPrimaryKeyExact,
	attributeNatural: attributeNatural,
	orderGroupBy: orderGroupBy,
	orderBy: orderBy,
	attributeSetInFilter: attributeSetInFilter,
	attributeSetExact: attributeSetExact,
	entityGroupProperty: entityGroupProperty,
	entityPrimaryKeyNatural: entityPrimaryKeyNatural,
	random: random,
	hierarchyOfReference: hierarchyOfReference,
	priceType: priceType,
	dataInLocales: dataInLocales,
	hierarchyOfSelf: hierarchyOfSelf,
	fromNode: fromNode,
	distance: distance,
	stopAt: stopAt,
	facetSummaryOfReference: facetSummaryOfReference,
	level: level,
	facetGroupsDisjunction: facetGroupsDisjunction,
	children: children,
	facetGroupsConjunction: facetGroupsConjunction,
	fromRoot: fromRoot,
	queryTelemetry: queryTelemetry,
	priceHistogram: priceHistogram,
	associatedDataContent: associatedDataContent,
	hierarchyContent: hierarchyContent,
	statistics: statistics,
	facetGroupsNegation: facetGroupsNegation,
	entityFetch: entityFetch,
	require: require,
	entityGroupFetch: entityGroupFetch,
	node: node,
	attributeContent: attributeContent,
	strip: strip,
	siblings: siblings,
	priceContent: priceContent,
	page: page,
	parents: parents,
	referenceContent: referenceContent,
	attributeHistogram: attributeHistogram,
	facetSummary: facetSummary
};

class EvitaQLQueryMode {
    constructor() { }
}
class EvitaQLConstraintListMode {
    constructor(listType) {
        this.listType = listType;
    }
}
var ConstraintListType;
(function (ConstraintListType) {
    ConstraintListType["Filter"] = "FILTER";
    ConstraintListType["Order"] = "ORDER";
    ConstraintListType["Require"] = "REQUIRE";
})(ConstraintListType || (ConstraintListType = {}));

function evitaQLCompletion(lang, config) {
    var _a;
    let completionList;
    if (config.mode instanceof EvitaQLQueryMode) {
        completionList = [
            createCompletion('query', 'Query is the root construct for querying data.'),
            ...Object.keys(constraints).map(it => createCompletion(it))
        ];
    }
    else if (config.mode instanceof EvitaQLConstraintListMode) {
        let constraintKeys;
        if (config.mode.listType === ConstraintListType.Filter) {
            constraintKeys = Object.keys(constraints).filter(it => {
                const constraint = constraints[it];
                return constraint.type === ConstraintListType.Filter && it !== 'filterBy';
            });
        }
        else if (config.mode.listType === ConstraintListType.Order) {
            constraintKeys = Object.keys(constraints).filter(it => {
                const constraint = constraints[it];
                return constraint.type === ConstraintListType.Order && it !== 'orderBy';
            });
        }
        else if (config.mode.listType === ConstraintListType.Require) {
            constraintKeys = Object.keys(constraints).filter(it => {
                const constraint = constraints[it];
                return constraint.type === ConstraintListType.Require && it !== 'require';
            });
        }
        else {
            throw new Error(`Unsupported constraint list type '${config.mode.listType}'`);
        }
        completionList = constraintKeys.map(it => createCompletion(it));
    }
    else {
        throw new Error(`Unsupported mode '${(_a = config.mode) === null || _a === void 0 ? void 0 : _a.toString()}'`);
    }
    return lang.data.of({
        autocomplete: completeFromList(completionList)
    });
}
function createCompletion(label, info) {
    const constraintDefinition = constraints[label];
    return {
        label,
        detail: '(...)',
        type: 'function',
        info: info ? info : constraintDefinition['shortDescription'] + "\n\n[Check detailed documentation](" + constraintDefinition['userDocsLink'] + ")",
        apply: (view, completion, from, to) => {
            view.dispatch({
                changes: { from, to, insert: label + '()' },
                selection: { anchor: from + label.length + 1 }
            });
        }
    };
}

const evitaQLLinter = linter(view => {
    const diagnostics = [];
    syntaxTree(view.state).cursor().iterate(node => {
        if (node.type.isError) {
            diagnostics.push({
                from: node.from,
                to: node.to,
                severity: 'error',
                message: 'Syntax error.'
            });
        }
    });
    return diagnostics;
});

// This file was generated by lezer-generator. You probably shouldn't edit it.
const parser = LRParser.deserialize({
  version: 14,
  states: "(WOYQPOOQ_QQOOOdQPO'#C_QOQPOOOiQPO'#CrQnQPOOOsQPO'#CbOOQO,58y,58yOOQO,59^,59^O#PQQO'#CdO_QQO'#CvQnQPOOOiQPO'#CcO#WQPO,58|O#`QQO'#DROOQO'#Co'#CoOOQO'#DQ'#DQOOQO'#DP'#DPOOQO,59O,59OO#nQPO,59OOOQO,59b,59bOOQO-E6t-E6tOOQO,58},58}OsQPO'#CuO#vQPO1G.hOOQO1G.h1G.hO$OQPO,59mO$^QQO,59qO$cQQO,59rO$hQQO,59sO!RQQO'#CtO$mQPO1G.jOOQO1G.j1G.jOOQO,59a,59aOOQO-E6s-E6sOOQO7+$S7+$SO$uQPO1G/XO%aQPO1G/]O%fQPO1G/^O%kQPO1G/_O%pQPO1G/]O%xQPO1G/^O&QQPO1G/_OOQO,59`,59`OOQO-E6r-E6rOOQO7+$U7+$UOOQO7+$w7+$wOOQO7+$x7+$xOOQO7+$y7+$yO&YQPO7+$wO&_QPO7+$xO&dQPO7+$yOOQO<<Hc<<HcOOQO<<Hd<<HdOOQO<<He<<He",
  stateData: "&x~OmOSPOS~OnQO~O|SO~OTUO~OTXO~O}YO~Oo[Op[Oq[Or[O~OX`OY`OZ`O[`O]`O^`O_`O``Oa`Ob`Od`Oe`Ov^O|SO~OSbO~P!ROSiO}gO~O[mO]lObkOwjO~OSpO}nO~OSsO}gO~O[wO]vObuOxtO~OwxO~OwyO~OwzO~OS}O}nO~OSuiSyiSziS{i}ui}yi}zi}{i~Ox!OO~Ox!PO~Ox!QO~Ob!ROx!OO~O]!SOx!PO~O[!TOx!QO~Ox!UO~Ox!VO~Ox!WO~O|Zba_`][^deYX]~",
  goto: "#a|PPP}PP!Q!T!ZPPPPPPPPPP!aPP!eP!o!u!{PPPPPPPP#R#X#]PPP#]#]#]RRORVQQ]URqgQWSRf[T`XnQTPSaXnRdYQocR|oQh]RrhQZTReZQcXR{nTaXnT_Xn",
  nodeNames: "⚠ Comment QueryRequest Query ArgsClosing ArgsOpening QueryBody RootConstraint ConstraintBody PositionalParameter NamedParameter String Int Float Boolean Date Time DateTime OffsetDateTime Range Uuid Enum Constraint ConstraintListRequest",
  maxTerm: 45,
  nodeProps: [
    ["openedBy", 4,"ArgsOpening"],
    ["closedBy", 5,"ArgsClosing"]
  ],
  skippedNodes: [0,1],
  repeatNodeCount: 3,
  tokenData: "#B]~RuXY#fYZ#f]^#fpq#frs#wwx&axy)iyz)n|})s}!O)z!P!Q*k!Q![+Y!a!b4c!b!c4h!c!}4|!}#O5_#P#Q5d#T#U5i#U#V3P#V#WLl#W#X!$z#X#Y!,S#Y#Z!7c#Z#[3P#[#]!Cf#]#`3P#`#a!Ke#a#b3P#b#c!MP#c#d!M{#d#e#!W#e#f#,O#f#g#.w#g#h#9S#h#i#@O#i#j#@t#j#o3P~#kSm~XY#fYZ#f]^#fpq#f~#zWpq#wqr#wrs$ds#O#w#O#P$i#P;'S#w;'S;=`&Z<%lO#w~$iOZ~~$lXrs#w!P!Q#w#O#P#w#U#V#w#Y#Z#w#b#c#w#f#g#w#h#i#w#i#j%X~%[R!Q![%e!c!i%e#T#Z%e~%hR!Q![%q!c!i%q#T#Z%q~%tR!Q![%}!c!i%}#T#Z%}~&QR!Q![#w!c!i#w#T#Z#w~&^P;=`<%l#w~&dXpq&aqr&asw&awx'Px#O&a#O#P'q#P;'S&a;'S;=`)c<%lO&a~'UXZ~pq&aqr&asw&awx'Px#O&a#O#P'q#P;'S&a;'S;=`)c<%lO&a~'tXrs&a!P!Q&a#O#P&a#U#V&a#Y#Z&a#b#c&a#f#g&a#h#i&a#i#j(a~(dR!Q![(m!c!i(m#T#Z(m~(pR!Q![(y!c!i(y#T#Z(y~(|R!Q![)V!c!i)V#T#Z)V~)YR!Q![&a!c!i&a#T#Z&a~)fP;=`<%l&a~)nOT~~)sOS~R)zOwQ}P~)}P!Q![*Q~*VQ[~!O!P*]!Q![*Q~*`P!Q![*c~*hP]~!Q![*c~*nP!P!Q*q~*vSP~OY*qZ;'S*q;'S;=`+S<%lO*q~+VP;=`<%l*q~+_S[~}!O+k!O!P*]!Q![-b#T#o3PQ+nQ!Q![+t#T#o+tQ+wR}!O,Q!Q![+t#T#o+tQ,TQ!Q![,Z#T#o,ZQ,^R}!O,g!Q![,Z#T#o,ZQ,jQ!Q![,p#T#o,pQ,sR}!O,|!Q![,p#T#o,pQ-PQ!Q![-V#T#o-VQ-[QdQ!Q![-V#T#o-V~-gT[~}!O+k!O!P*]!Q![-v![!]3]#T#o3P~-{S[~}!O+k!O!P*]!Q![.X#T#o3P~.^S[~}!O.j!O!P*]!Q![2n#T#o3P~.mQ!Q![.s#T#o+t~.vR}!O,Q!Q![/P#T#o+t~/SR}!O/]!Q![+t#T#o+t~/`Q!Q![/f#T#o,Z~/iR}!O,g!Q![/r#T#o,Z~/wS_~}!O,g!Q![,Z!v!w0T#T#o,Z~0WP!Q![0Z~0^P!Q![0a~0dP![!]0g~0jP!Q![0m~0pP!Q![0s~0vP![!]0y~0|P!Q![1P~1SP!Q![1V~1[Ra~{|1e}!O1e!O!P2Y~1hP!Q![1k~1nP!Q![1q~1tP![!]1w~1zP!Q![1}~2QP!Q![2T~2YOb~~2]P!Q![2`~2eRa~{|1e}!O1e!Q![2`~2sS[~}!O+k!O!P*]!Q![2n#T#o3PQ3SR}!O+k!Q![3P#T#o3P~3`P!Q![3c~3fP!Q![3i~3lP![!]3o~3rP!Q![3u~3xP!Q![3{~4QP`~!O!P4T~4WP!Q![4Z~4`P`~!Q![4Z~4hOX~~4kP#T#o4n~4sRY~!Q![4n!c!}4n#T#o4n~5RQe~!c!}4|#R#S5X~5[P!c!}4|~5dOv~~5iOx~Q5lW}!O+k!Q![3P#T#b3P#b#c6U#c#g3P#g#h6v#h#i<R#i#o3PQ6XT}!O+k!Q![3P#T#W3P#W#X6h#X#o3PQ6mR|Q}!O+k!Q![3P#T#o3PQ6yT}!O+k!Q![3P#T#g3P#g#h7Y#h#o3PQ7]T}!O+k!Q![3P#T#c3P#c#d7l#d#o3PQ7oT}!O+k!Q![3P#T#V3P#V#W8O#W#o3PQ8RT}!O+k!Q![3P#T#]3P#]#^8b#^#o3PQ8eS}!O+k!Q![3P#T#U8q#U#o3PQ8tT}!O+k!Q![3P#T#h3P#h#i9T#i#o3PQ9WT}!O+k!Q![3P#T#X3P#X#Y9g#Y#o3PQ9jT}!O+k!Q![3P#T#W3P#W#X9y#X#o3PQ9|S}!O+k!Q![3P!f!g:Y#T#o3PQ:]P#T#U:`Q:cP#h#i:fQ:iP#T#U:lQ:oP!e!f:rQ:uP#c#d:xQ:{P#b#c;OQ;RP#h#i;UQ;XP#X#Y;[Q;_P#b#c;bQ;eP#h#i;hQ;mP|Q!c!d;pQ;sP#`#a;vQ;yP#`#a;|Q<RO|QQ<UT}!O+k!Q![3P#T#h3P#h#i<e#i#o3PQ<hT}!O+k!Q![3P#T#f3P#f#g<w#g#o3PQ<zT}!O+k!Q![3P#T#]3P#]#^=Z#^#o3PQ=^T}!O+k!Q![3P#T#U3P#U#V=m#V#o3PQ=pT}!O+k!Q![3P#T#i3P#i#j>P#j#o3PQ>ST}!O+k!Q![3P#T#h3P#h#i>c#i#o3PQ>fT}!O+k!Q![3P#T#X3P#X#Y>u#Y#o3PQ>x[}!O+k!Q![3P!d!e?n!e!f@d!g!hAc!i!jC{!j!kE{!k!lF}!n!oI`!p!qIr!u!vJb#T#o3PQ?qP#X#Y?tQ?wP#h#i?zQ?}P#k#l@QQ@TP#X#Y@WQ@ZP#X#Y@^Q@aP#b#c;|Q@gP#c#d@jQ@mP#b#c@pQ@sP#h#i@vQ@yQ#T#UAP#X#Y;[QASP#]#^AVQAYP#b#cA]QA`P#g#h;|QAfQ#b#cAl#e#fBbQAoP#W#XArQAuP#g#hAxQA{P!y!zBOQBRP#]#^BUQBXP#h#iB[QB_P#[#];|QBeP#i#jBhQBkP#T#UBnQBqP#`#aBtQBwP#g#hBzQCPQ|Q!h!iCV!v!wCoQCYP#T#UC]QC`P#`#aCcQCfP#g#hCiQClP#X#Y;|QCrP#f#gCuQCxP#i#jCiQDOP#f#gDRQDUP#X#YDXQD[P#T#UD_QDbP#h#iDeQDhP#X#YDkQDnP#f#gDqQDtP!v!wDwQDzP#[#]D}QEQP#T#UETQEWP#b#cEZQE`P|Q!g!hEcQEfP#e#fEiQElP#i#jEoQErP#T#UEuQExP#`#aA]QFOP#]#^FRQFUP#g#hFXQF[P#h#iF_QFbP#c#dFeQFhP#Z#[FkQFnP#f#gFqQFtP#T#UFwQFzP#a#b;|QGQQ#b#cGW#g#hHkQGZQ!t!uGa!u!vH_QGdP#T#UGgQGjP#b#cGmQGpP#Z#[GsQGvP#X#YGyQHOP|Q!p!qHRQHUP#c#dHXQH[P#k#l;|QHbP#X#YHeQHhP#h#i;|QHpP|Q!p!qHsQHvQ#c#dH|#i#j;pQIPP#h#iISQIVP!p!qIYQI]P#i#j;pQIcP#X#YIfQIiP#g#hIlQIoP#g#hDqQIuP#T#UIxQI{P#h#iJOQJRP#i#jJUQJXP#f#gJ[QJ_P#T#U;vQJeQ#X#YJk#h#iLYQJnP#h#iJqQJtQ!g!hJz!k!lK^QJ}P#l#mKQQKTP#T#UKWQKZP#V#WHeQKaP#b#cKdQKgP!h!iKjQKmP#]#^KpQKsP#`#aKvQKyP#h#iK|QLPP#X#YLSQLVP#f#g;|QL]P#T#UL`QLcP#f#gLfQLiP#h#iArRLoV}!O+k!Q![3P#T#[3P#[#]MU#]#c3P#c#d! f#d#o3PQMXT}!O+k!Q![3P#T#]3P#]#^Mh#^#o3PQMkT}!O+k!Q![3P#T#`3P#`#aMz#a#o3PQM}T}!O+k!Q![3P#T#W3P#W#XN^#X#o3PQNaT}!O+k!Q![3P#T#f3P#f#gNp#g#o3PQNsT}!O+k!Q![3P#T#X3P#X#Y! S#Y#o3PQ! VT}!O+k!Q![3P#T#b3P#b#c6h#c#o3PR! iT}!O+k!Q![3P#T#`3P#`#a! x#a#o3PR! {T}!O+k!Q![3P#T#`3P#`#a!![#a#o3PR!!_T}!O+k!Q![3P#T#X3P#X#Y!!n#Y#o3PR!!qT}!O+k!Q![3P#T#V3P#V#W!#Q#W#o3PR!#TT}!O+k!Q![3P#T#h3P#h#i!#d#i#o3PR!#gT}!O+k!Q![3P#T#]3P#]#^!#v#^#o3PR!#yT}!O+k!Q![3P#T#c3P#c#d!$Y#d#o3PR!$]T}!O+k!Q![3P#T#b3P#b#c!$l#c#o3PR!$qRoP}!O+k!Q![3P#T#o3PQ!$}U}!O+k!Q![3P#T#U!%a#U#]3P#]#^!'e#^#o3PQ!%dT}!O+k!Q![3P#T#h3P#h#i!%s#i#o3PQ!%vS}!O+k!Q![3P#T#U!&S#U#o3PQ!&VS}!O+k!Q![3P!k!l!&c#T#o3PQ!&fP#b#c!&iQ!&lP!n!o!&oQ!&rP#c#d!&uQ!&xP#V#W!&{Q!'OP#T#U!'RQ!'UP#`#a!'XQ!'[P#X#Y!'_Q!'bP#g#h;hQ!'hU}!O+k!Q![3P#T#f3P#f#g!'z#g#h!*X#h#o3PQ!'}T}!O+k!Q![3P#T#X3P#X#Y!(^#Y#o3PQ!(aT}!O+k!Q![3P#T#V3P#V#W!(p#W#o3PQ!(sT}!O+k!Q![3P#T#h3P#h#i!)S#i#o3PQ!)VS}!O+k!Q![3P!t!u!)c#T#o3PQ!)fP#X#Y!)iQ!)lP#`#a!)oQ!)rP#T#U!)uQ!)xP#h#i!){Q!*OP#]#^!*RQ!*UP#c#d@^Q!*[T}!O+k!Q![3P#T#h3P#h#i!*k#i#o3PQ!*nS}!O+k!Q![3P#T#U!*z#U#o3PQ!*}T}!O+k!Q![3P#T#b3P#b#c!+^#c#o3PQ!+aT}!O+k!Q![3P#T#V3P#V#W!+p#W#o3PQ!+sT}!O+k!Q![3P#T#X3P#X#Y6h#Y#o3PQ!,VV}!O+k!Q![3P#T#b3P#b#c!,l#c#l3P#l#m!4Q#m#o3PQ!,oT}!O+k!Q![3P#T#h3P#h#i!-O#i#o3PQ!-RT}!O+k!Q![3P#T#]3P#]#^!-b#^#o3PQ!-eT}!O+k!Q![3P#T#h3P#h#i!-t#i#o3PQ!-wT}!O+k!Q![3P#T#m3P#m#n!.W#n#o3PQ!.ZW}!O+k!Q![3P!h!i!.s!i!j!/V!j!k!0t!n!o!1d!r!s!2Y#T#o3PQ!.vP#X#Y!.yQ!.|P#h#i!/PQ!/SP#V#WB[Q!/YP#f#g!/]Q!/`P#c#d!/cQ!/fP#i#j!/iQ!/lP#d#e!/oQ!/rQ!h!i!.s!r!s!/xQ!/{P#f#g!0OQ!0RP#c#d!0UQ!0XP#d#e!0[Q!0_P#X#Y!0bQ!0eP#f#g!0hQ!0kP#h#i!0nQ!0qP#m#n;|Q!0wP#T#U!0zQ!0}P#j#k!1QQ!1TP#]#^!1WQ!1ZP#b#c!1^Q!1aP#Z#[;|Q!1gP#c#d!1jQ!1mP#V#W!1pQ!1sP#T#U!1vQ!1yP#`#a!1|Q!2PP#X#Y!2SQ!2VP!g!hEcQ!2]P#f#g!2`Q!2cQ#]#^!2i#c#d!0UQ!2lP#a#b!2oQ!2rP#T#U!2uQ!2xP#f#g!2{Q!3OP#m#n!3RQ!3UP!m!n!3XQ!3[P#X#Y!3_Q!3bP#m#n!3eQ!3hR!g!hJz!k!l!3q!p!qIrQ!3tP#b#c!3wQ!3zQ!h!iKj!u!vH_Q!4TT}!O+k!Q![3P#T#V3P#V#W!4d#W#o3PQ!4gT}!O+k!Q![3P#T#`3P#`#a!4v#a#o3PQ!4yT}!O+k!Q![3P#T#i3P#i#j!5Y#j#o3PQ!5]T}!O+k!Q![3P#T#W3P#W#X!5l#X#o3PQ!5oT}!O+k!Q![3P#T#]3P#]#^!6O#^#o3PQ!6RT}!O+k!Q![3P#T#b3P#b#c!6b#c#o3PQ!6eT}!O+k!Q![3P#T#Z3P#Z#[!6t#[#o3PQ!6yS|Q}!O+k!Q![3P!t!u!7V#T#o3PQ!7YP#c#d!7]Q!7`P#c#dHe~!7fW}!O+k!Q![3P#T#U!8O#U#]3P#]#^!?X#^#f3P#f#g!BQ#g#o3P~!8RV}!O+k!Q![3P#T#V3P#V#W!8h#W#`3P#`#a!>T#a#o3PQ!8kT}!O+k!Q![3P#T#X3P#X#Y!8z#Y#o3PQ!8}T}!O+k!Q![3P#T#h3P#h#i!9^#i#o3PQ!9aU}!O+k!Q![3P!i!j!9s!j!k!0t!u!v!;}#T#o3PQ!9vP#f#g!9yQ!9|P#c#d!:PQ!:SP#i#j!:VQ!:YP#d#e!:]Q!:`P#g#h!:cQ!:fR!e!f!:o!f!g!;e!p!q!;qQ!:rP#c#d!:uQ!:xP#b#c!:{Q!;OP#^#_!;RQ!;UP#i#j!;XQ!;[P#b#c!;_Q!;bP#V#W!)uQ!;hP#]#^!;kQ!;nP#g#h!:{Q!;tP#X#Y!;wQ!;zP#Z#[!)oQ!<QP#i#j!<TQ!<WP#a#b!<ZQ!<^P#a#b!<aQ!<dP#T#U!<gQ!<jP#f#g!<mQ!<pP#m#n!<sQ!<xP|Q!q!r!<{Q!=OP#Y#Z!=RQ!=UP!t!u!=XQ!=[P#X#Y!=_Q!=bP#Y#Z!=eQ!=hP#X#Y!=kQ!=nP#f#g!=qQ!=tP#X#Y!=wQ!=zP#b#c!=}Q!>QP#V#WCi~!>WT}!O+k!Q![3P#T#g3P#g#h!>g#h#o3P~!>jT}!O+k!Q![3P#T#X3P#X#Y!>y#Y#o3P~!?OR^~}!O+k!Q![3P#T#o3P~!?[T}!O+k!Q![3P#T#`3P#`#a!?k#a#o3P~!?nT}!O+k!Q![3P#T#h3P#h#i!?}#i#o3P~!@QT}!O+k!Q![3P#T#X3P#X#Y!@a#Y#o3P~!@dT}!O+k!Q![3P#T#f3P#f#g!@s#g#o3P~!@vT}!O+k!Q![3P!d!e!AV!i!j!Ab#T#o3P~!AYP#m#n!A]~!AbOp~Q!AeP#f#g!AhQ!AkP#c#d!AnQ!AqP#i#j!AtQ!AwP#d#e!AzQ!A}P!d!e!0nQ!BTT}!O+k!Q![3P#T#c3P#c#d!Bd#d#o3PQ!BgT}!O+k!Q![3P#T#a3P#a#b!Bv#b#o3PQ!ByT}!O+k!Q![3P!p!q!CY!t!u!7V#T#o3PQ!C]P#c#d!C`Q!CcP#W#XCiQ!CiU}!O+k!Q![3P#T#U!C{#U#]3P#]#^!Eg#^#o3PQ!DOT}!O+k!Q![3P#T#j3P#j#k!D_#k#o3PQ!DbT}!O+k!Q![3P#T#]3P#]#^!Dq#^#o3PQ!DtT}!O+k!Q![3P#T#b3P#b#c!ET#c#o3PQ!EWT}!O+k!Q![3P#T#Z3P#Z#[6h#[#o3PQ!EjT}!O+k!Q![3P#T#X3P#X#Y!Ey#Y#o3PQ!E|T}!O+k!Q![3P#T#f3P#f#g!F]#g#o3PQ!F`S}!O+k!Q![3P#T#U!Fl#U#o3PQ!FoT}!O+k!Q![3P#T#f3P#f#g!GO#g#o3PQ!GRT}!O+k!Q![3P#T#V3P#V#W!Gb#W#o3PQ!GeT}!O+k!Q![3P#T#[3P#[#]!Gt#]#o3PQ!GwT}!O+k!Q![3P#T#m3P#m#n!HW#n#o3PQ!HZU}!O+k!Q![3P!e!f!Hm!q!r!I]!y!z!JO#T#o3PQ!HpP#c#d!HsQ!HvP#b#c!HyQ!H|P#h#i!IPQ!ISP#X#Y!IVQ!IYP#b#cHeQ!I`P#Y#Z!IcQ!IfQ!t!u!=X!u!v!IlQ!IoP#X#Y!IrQ!IuP#`#a!IxQ!I{P#Y#Z;|Q!JRP#]#^!JUQ!JXP#h#i!J[Q!J_P#[#]!JbQ!JeP#]#^!JhQ!JkP#b#c!JnQ!JsQ|Q!t!u!Jy!u!v!IlQ!J|P#c#d!KPQ!KSP#c#d!KVQ!KYP#h#i!K]Q!KbP|Q!u!v!IlQ!KhT}!O+k!Q![3P#T#X3P#X#Y!Kw#Y#o3PQ!KzT}!O+k!Q![3P#T#j3P#j#k!LZ#k#o3PQ!L^T}!O+k!Q![3P#T#X3P#X#Y!Lm#Y#o3PQ!LpT}!O+k!Q![3P#T#`3P#`#a6h#a#o3PQ!MST}!O+k!Q![3P#T#c3P#c#d!Mc#d#o3PQ!MfV}!O+k!Q![3P#T#W3P#W#X!+p#X#h3P#h#i6h#i#o3PR!NOT}!O+k!Q![3P#T#f3P#f#g!N_#g#o3PR!NdT|Q}!O+k!Q![3P#T#W3P#W#X!Ns#X#o3PR!NvT}!O+k!Q![3P#T#X3P#X#Y# V#Y#o3PR# YT}!O+k!Q![3P#T#f3P#f#g# i#g#o3PR# lT}!O+k!Q![3P!d!e# {!i!j!Ab#T#o3PP#!OP#m#n#!RP#!WOqPQ#!ZU}!O+k!Q![3P#T#U#!m#U#f3P#f#g#$q#g#o3PQ#!pV}!O+k!Q![3P#T#Z3P#Z#[!+p#[#f3P#f#g##V#g#o3PQ##YT}!O+k!Q![3P#T#X3P#X#Y##i#Y#o3PQ##lT}!O+k!Q![3P#T#b3P#b#c##{#c#o3PQ#$OT}!O+k!Q![3P#T#h3P#h#i#$_#i#o3PQ#$bT}!O+k!Q![3P#T#g3P#g#h6h#h#o3PQ#$tT}!O+k!Q![3P#T#]3P#]#^#%T#^#o3PQ#%WT}!O+k!Q![3P#T#V3P#V#W#%g#W#o3PQ#%jT}!O+k!Q![3P#T#X3P#X#Y#%y#Y#o3PQ#%|Y}!O+k!Q![3P!d!e?n!e!f#&l!j!kE{!k!l#(u!p!qIr!v!w#*|!x!y#+Y#T#o3PQ#&oP#c#d#&rQ#&uP#b#c#&xQ#&{P#h#i#'OQ#'RP#X#Y#'UQ#'XP#b#c#'[Q#'_P#h#i#'bQ#'gQ|Q!c!d;p!t!u#'mQ#'pP#X#Y#'sQ#'vP#g#h#'yQ#'|P#d#e#(PQ#(SP#X#Y#(VQ#(YP#V#W#(]Q#(`P#h#i#(cQ#(fP#]#^#(iQ#(lP#b#c#(oQ#(rP#Z#[KdQ#(xP#b#c#({Q#)OQ!e!f#)U!r!s#)zQ#)XP#i#j#)[Q#)_P#f#g#)bQ#)eP#f#g#)hQ#)kP#X#Y#)nQ#)qP#b#c#)tQ#)wP#V#W!0nQ#)}P#f#g#*QQ#*TP#]#^#*WQ#*ZP#V#W#*^Q#*aP#X#Y#*dQ#*gP!n!o#*jQ#*mP#]#^#*pQ#*sP#g#h#*vQ#*yP#h#iA]Q#+PP#m#n#+SQ#+VP#d#eCiQ#+]P#T#U#+`Q#+cP#`#a#+fQ#+iP#]#^#+lQ#+oP#W#X#+rQ#+uP!k!l#+xQ#+{P#b#cGyR#,RT}!O+k!Q![3P#T#i3P#i#j#,b#j#o3PR#,eT}!O+k!Q![3P#T#X3P#X#Y#,t#Y#o3PR#,wT}!O+k!Q![3P#T#f3P#f#g#-W#g#o3PR#-ZT}!O+k!Q![3P#T#m3P#m#n#-j#n#o3PR#-oSnP}!O+k!Q![3P!v!w#-{#T#o3PQ#.OP#X#Y#.RQ#.UP#`#a#.XQ#.[P#X#Y#._Q#.bP#a#b#.eQ#.hP#X#Y#.kQ#.nP#h#i#.qQ#.tP#f#g!0nR#.zU}!O+k!Q![3P#T#U#/^#U#X3P#X#Y#0x#Y#o3PQ#/aT}!O+k!Q![3P#T#b3P#b#c#/p#c#o3PQ#/sT}!O+k!Q![3P#T#W3P#W#X#0S#X#o3PQ#0VT}!O+k!Q![3P#T#c3P#c#d#0f#d#o3PQ#0iT}!O+k!Q![3P#T#a3P#a#b6h#b#o3PR#0{V}!O+k!Q![3P#T#Y3P#Y#Z#1b#Z#e3P#e#f#7Y#f#o3PQ#1eT}!O+k!Q![3P#T#X3P#X#Y#1t#Y#o3PQ#1wT}!O+k!Q![3P#T#f3P#f#g#2W#g#o3PQ#2ZT}!O+k!Q![3P#T#X3P#X#Y#2j#Y#o3PQ#2mT}!O+k!Q![3P#T#b3P#b#c#2|#c#o3PQ#3PT}!O+k!Q![3P#T#V3P#V#W#3`#W#o3PQ#3cT}!O+k!Q![3P#T#X3P#X#Y#3r#Y#o3PQ#3uU}!O+k!Q![3P!e!f#4X!j!k!0t!r!s!/x#T#o3PQ#4[P#c#d#4_Q#4bP#b#c#4eQ#4hP#h#i#4kQ#4nP#X#Y#4qQ#4tP#b#c#4wQ#4zP#h#i#4}Q#5SQ|Q!c!d#5Y!y!z#5nQ#5]P#`#a#5`Q#5cP#`#a#5fQ#5kP|Q!y!z#5nQ#5qP#]#^#5tQ#5wP#h#i#5zQ#5}P#[#]#6QQ#6TP!c!d#6WQ#6ZP#h#i#6^Q#6aP#h#i#6dQ#6gP#f#g#6jQ#6mP#]#^#6pQ#6sP#U#V#6vQ#6yP#i#j#6|Q#7PP#h#i#7SQ#7VP#X#YA]R#7]T}!O+k!Q![3P#T#i3P#i#j#7l#j#o3PR#7oT}!O+k!Q![3P#T#]3P#]#^#8O#^#o3PR#8RT}!O+k!Q![3P#T#f3P#f#g#8b#g#o3PR#8eT}!O+k!Q![3P#T#X3P#X#Y#8t#Y#o3PR#8yRrP}!O+k!Q![3P#T#o3PQ#9VV}!O+k!Q![3P#T#]3P#]#^#9l#^#h3P#h#i#;j#i#o3PQ#9oT}!O+k!Q![3P#T#U3P#U#V#:O#V#o3PQ#:RT}!O+k!Q![3P#T#`3P#`#a#:b#a#o3PQ#:eT}!O+k!Q![3P#T#]3P#]#^#:t#^#o3PQ#:wT}!O+k!Q![3P#T#b3P#b#c#;W#c#o3PQ#;ZT}!O+k!Q![3P#T#Z3P#Z#[#$_#[#o3PQ#;mW}!O+k!Q![3P#T#U#<V#U#c3P#c#d#>g#d#f3P#f#g#?Y#g#o3PQ#<YT}!O+k!Q![3P#T#h3P#h#i#<i#i#o3PQ#<lT}!O+k!Q![3P#T#]3P#]#^#<{#^#o3PQ#=OT}!O+k!Q![3P#T#g3P#g#h#=_#h#o3PQ#=bT}!O+k!Q![3P#T#h3P#h#i#=q#i#o3PQ#=tT}!O+k!Q![3P#T#]3P#]#^#>T#^#o3PQ#>WT}!O+k!Q![3P#T#V3P#V#W#$_#W#o3PQ#>jT}!O+k!Q![3P#T#d3P#d#e#>y#e#o3PQ#>|S}!O+k!Q![3P!c!dHe#T#o3PQ#?]T}!O+k!Q![3P#T#]3P#]#^#?l#^#o3PQ#?oT}!O+k!Q![3P#T#d3P#d#e6h#e#o3P~#@RT}!O+k!Q![3P#T#f3P#f#g#@b#g#o3P~#@eT}!O+k!Q![3P#T#i3P#i#j!>g#j#o3PQ#@wT}!O+k!Q![3P#T#g3P#g#h#AW#h#o3PQ#AZT}!O+k!Q![3P#T#X3P#X#Y#Aj#Y#o3PQ#AmT}!O+k!Q![3P#T#f3P#f#g#A|#g#o3PQ#BPS}!O+k!Q![3P!h!iKj#T#o3P",
  tokenizers: [0, 1],
  topRules: {"QueryRequest":[0,2],"ConstraintListRequest":[1,23]},
  tokenPrec: 255
});

function evitaQLLanguage(config) {
    var _a;
    let top;
    if (config.mode instanceof EvitaQLQueryMode) {
        top = 'QueryRequest';
    }
    else if (config.mode instanceof EvitaQLConstraintListMode) {
        top = 'ConstraintListRequest';
    }
    else {
        throw new Error(`Unsupported mode '${(_a = config.mode) === null || _a === void 0 ? void 0 : _a.toString()}'`);
    }
    return LRLanguage.define({
        parser: parser.configure({
            top,
            props: [
                indentNodeProp.add({
                    "QueryBody ConstraintBody": delimitedIndent({ closing: ')', align: true }),
                    Range: delimitedIndent({ closing: ']', align: true })
                }),
                foldNodeProp.add({
                    'QueryBody ConstraintBody': foldInside
                }),
                styleTags({
                    String: tags.string,
                    Int: tags.integer,
                    Float: tags.float,
                    Boolean: tags.bool,
                    Date: tags.literal,
                    Time: tags.literal,
                    DateTime: tags.literal,
                    OffsetDateTime: tags.literal,
                    Uuid: tags.literal,
                    Enum: tags.constant(tags.variableName),
                    Query: tags.function(tags.variableName),
                    RootConstraint: tags.function(tags.variableName),
                    Constraint: tags.function(tags.variableName),
                    Comment: tags.lineComment,
                    ',': tags.separator,
                    '( )': tags.paren,
                    '[ ]': tags.squareBracket
                })
            ]
        }),
        languageData: {
            commentTokens: { line: '//' }
        }
    });
}

function evitaQL(config = { mode: new EvitaQLQueryMode() }) {
    const lang = evitaQLLanguage(config);
    return new LanguageSupport(lang, [evitaQLCompletion(lang, config), evitaQLLinter]);
}

export { ConstraintListType, EvitaQLConstraintListMode, EvitaQLQueryMode, evitaQL, evitaQLCompletion, evitaQLLanguage, evitaQLLinter };
